package ij.plugin;

import ij.IJ;
import ij.ImagePlus;
import ij.Prefs;
import ij.WindowManager;
import ij.gui.DialogListener;
import ij.gui.GenericDialog;
import ij.gui.NonBlockingGenericDialog;
import ij.gui.Roi;
import ij.gui.Toolbar;
import ij.plugin.frame.PlugInFrame;

import java.awt.AWTEvent;
import java.awt.Color;
import java.util.Vector;

import javax.swing.JFrame;
import javax.swing.JTextField;
import javax.swing.JScrollBar;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import java.awt.event.AdjustmentListener;
import java.awt.event.AdjustmentEvent;
import ij.util.Tools;

/** This plugin implements the rounded rectangle tool dialog box. */
public class RectToolOptions extends PlugInFrame implements PlugIn, DialogListener, DocumentListener, AdjustmentListener {
	private String strokeColorName, title="Rounded Rectangle Tool";
	private static GenericDialog gd;
        private static double defaultStrokeWidth = 0.0;
        private static Color defaultStrokeColor;
        private static int cornerDiameter;
        private static JFrame frame;
        private Vector numbers, sliders, stringFields;

        public RectToolOptions(String title){
             super(title);
        }
        public RectToolOptions(){
            this("");
        }

 	public void run(String arg) {
 		if (gd!=null && gd.isVisible()){
                     gd.toFront();
                 }
 		else{
                     rectToolOptions();
                 }
	}

	void rectToolOptions() {
            if (defaultStrokeColor==null)
                    defaultStrokeColor = Roi.getColor();
            Color strokeColor = defaultStrokeColor;
            Color fillColor = null;
            if (defaultStrokeWidth==0.0)
                    defaultStrokeWidth = 1.0;
            double strokeWidth = defaultStrokeWidth;
            int cornerDiameter = (int)Prefs.get(Toolbar.CORNER_DIAMETER, 20);
            ImagePlus imp = WindowManager.getCurrentImage();
            Roi roi = imp!=null?imp.getRoi():null;
            if (roi!=null && (roi.getType()==Roi.RECTANGLE)) {
                    strokeColor = roi.getStrokeColor();
                    if (strokeColor==null)
                            strokeColor = Roi.getColor();
                    fillColor = roi.getFillColor();
                    strokeWidth = roi.getStrokeWidth();
                    cornerDiameter = roi.getCornerDiameter();
            }
            String strokec = Colors.colorToString(strokeColor);
            String fillc = Colors.colorToString(fillColor);

            gd = new NonBlockingGenericDialog("Rounded Rectangle Tool");
            gd.addSlider("Stroke width:", 1, 25, (int)strokeWidth);
            gd.addSlider("Corner diameter:", 0, 200, cornerDiameter);
            gd.addStringField("Stroke color: ", strokec);
            gd.addStringField("Fill color: ", fillc);
            gd.addDialogListener(this);
            numbers = gd.getNumericFields();
            for (int i=0; i<numbers.size(); i++)
                ((JTextField)numbers.elementAt(i)).getDocument().addDocumentListener(this);
            stringFields = gd.getStringFields();
            for (int i=0; i<stringFields.size(); i++)
                ((JTextField)stringFields.elementAt(i)).getDocument().addDocumentListener(this);
            sliders = gd.getSliders();
            for (int i=0; i<sliders.size(); i++)
                ((JScrollBar)sliders.elementAt(i)).addAdjustmentListener(this);
		gd.showDialog();
                defaultStrokeWidth = Tools.parseDouble(((JTextField)numbers.elementAt(0)).getText());
                cornerDiameter = (int)Tools.parseDouble(((JTextField)numbers.elementAt(1)).getText());
	}

	public boolean dialogItemChanged(GenericDialog gd, AWTEvent e) {
            return true;
	}
        public void textValueChanged(DocumentEvent e) {
            double strokeWidth2 = Tools.parseDouble(((JTextField)numbers.elementAt(0)).getText());
            int cornerDiameter2 = (int)Tools.parseDouble(((JTextField)numbers.elementAt(1)).getText());
            String strokec2 = ((JTextField)stringFields.elementAt(0)).getText();
            //System.out.println("strokec2="+strokec2);
            String fillc2 = ((JTextField)stringFields.elementAt(1)).getText();
            //System.out.println("fillc2="+fillc2);
            ImagePlus imp = WindowManager.getCurrentImage();
            Roi roi = imp!=null?imp.getRoi():null;
            Color strokeColor = null;
            if (roi!=null && (roi.getType()==Roi.RECTANGLE)) {
                    roi.setStrokeWidth((int)strokeWidth2);
                    roi.setCornerDiameter((int)(cornerDiameter2));
                    strokeColor = Colors.decode(strokec2, roi.getStrokeColor());
                    Color fillColor = Colors.decode(fillc2, roi.getFillColor());
                    //System.out.println(strokeWidth2);
                    roi.setStrokeColor(strokeColor);
                    roi.setFillColor(fillColor);
            }
            defaultStrokeWidth = strokeWidth2;
            if (strokeColor!=null){
                Toolbar.setForegroundColor(strokeColor);
            }
            Toolbar.setRoundRectArcSize(cornerDiameter2);
            if (cornerDiameter2>0) {
                    if (!Toolbar.getToolName().equals("roundrect"))
                            IJ.setTool("roundrect");
            }
        }
        public void insertUpdate(DocumentEvent e){
            textValueChanged(e);
        }
        public void removeUpdate(DocumentEvent e){
            textValueChanged(e);
        }
        public void changedUpdate(DocumentEvent e){
        }
        public synchronized void adjustmentValueChanged(AdjustmentEvent e) {
                Object source = e.getSource();
                for (int i=0; i<sliders.size(); i++) {
                        if (source==sliders.elementAt(i)) {
                                JScrollBar sb = (JScrollBar)source;
                                JTextField tf = (JTextField)numbers.elementAt(i);
                                textValueChanged(null);
                        }
                }
        }

        public static Color getDefaultStrokeColor() {
                return defaultStrokeColor;
        }

        public static float getDefaultStrokeWidth() {
                return (float)defaultStrokeWidth;
        }

}
